<?php

namespace Btravel\Objects\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Objects\Http\Requests\ClubRequest;
use Btravel\Objects\Interfaces\ClubRepositoryInterface;
use Btravel\Objects\Models\Club;

/**
 * Resource controller class for club.
 */
class ClubResourceController extends BaseController
{

    /**
     * Initialize club resource controller.
     *
     * @param type ClubRepositoryInterface $club
     *
     * @return null
     */
    public function __construct(ClubRepositoryInterface $club)
    {
        parent::__construct();
        $this->repository = $club;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Objects\Repositories\Criteria\ClubResourceCriteria::class);
    }

    /**
     * Display a list of club.
     *
     * @return Response
     */
    public function index(ClubRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Objects\Repositories\Presenter\ClubPresenter::class)
                ->$function();
        }

        $clubs = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('objects::club.names'))
            ->view('objects::club.index', true)
            ->data(compact('clubs', 'view'))
            ->output();
    }

    /**
     * Display club.
     *
     * @param Request $request
     * @param Model   $club
     *
     * @return Response
     */
    public function show(ClubRequest $request, Club $club)
    {

        if ($club->exists) {
            $view = 'objects::club.show';
        } else {
            $view = 'objects::club.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('objects::club.name'))
            ->data(compact('club'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new club.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ClubRequest $request)
    {

        $club = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('objects::club.name')) 
            ->view('objects::club.create', true) 
            ->data(compact('club'))
            ->output();
    }

    /**
     * Create new club.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ClubRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $club                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('objects::club.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/club/' . $club->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/objects/club'))
                ->redirect();
        }

    }

    /**
     * Show club for editing.
     *
     * @param Request $request
     * @param Model   $club
     *
     * @return Response
     */
    public function edit(ClubRequest $request, Club $club)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('objects::club.name'))
            ->view('objects::club.edit', true)
            ->data(compact('club'))
            ->output();
    }

    /**
     * Update the club.
     *
     * @param Request $request
     * @param Model   $club
     *
     * @return Response
     */
    public function update(ClubRequest $request, Club $club)
    {
        try {
            $attributes = $request->all();

            $club->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('objects::club.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/club/' . $club->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/club/' . $club->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the club.
     *
     * @param Model   $club
     *
     * @return Response
     */
    public function destroy(ClubRequest $request, Club $club)
    {
        try {

            $club->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::club.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('objects/club/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/club/' . $club->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple club.
     *
     * @param Model   $club
     *
     * @return Response
     */
    public function delete(ClubRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::club.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('objects/club'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/club'))
                ->redirect();
        }

    }

    /**
     * Restore deleted clubs.
     *
     * @param Model   $club
     *
     * @return Response
     */
    public function restore(ClubRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('objects::club.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/objects/club'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/club/'))
                ->redirect();
        }

    }

}
