<?php

namespace Btravel\Objects\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Objects\Http\Requests\HotelRequest;
use Btravel\Objects\Interfaces\HotelRepositoryInterface;
use Btravel\Objects\Models\Hotel;

/**
 * Resource controller class for hotel.
 */
class HotelResourceController extends BaseController
{

    /**
     * Initialize hotel resource controller.
     *
     * @param type HotelRepositoryInterface $hotel
     *
     * @return null
     */
    public function __construct(HotelRepositoryInterface $hotel)
    {
        parent::__construct();
        $this->repository = $hotel;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Objects\Repositories\Criteria\HotelResourceCriteria::class);
    }

    /**
     * Display a list of hotel.
     *
     * @return Response
     */
    public function index(HotelRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Objects\Repositories\Presenter\HotelPresenter::class)
                ->$function();
        }

        $hotels = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('objects::hotel.names'))
            ->view('objects::hotel.index', true)
            ->data(compact('hotels', 'view'))
            ->output();
    }

    /**
     * Display hotel.
     *
     * @param Request $request
     * @param Model   $hotel
     *
     * @return Response
     */
    public function show(HotelRequest $request, Hotel $hotel)
    {

        if ($hotel->exists) {
            $view = 'objects::hotel.show';
        } else {
            $view = 'objects::hotel.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('objects::hotel.name'))
            ->data(compact('hotel'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new hotel.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(HotelRequest $request)
    {

        $hotel = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('objects::hotel.name')) 
            ->view('objects::hotel.create', true) 
            ->data(compact('hotel'))
            ->output();
    }

    /**
     * Create new hotel.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(HotelRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $hotel                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('objects::hotel.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/hotel/' . $hotel->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/objects/hotel'))
                ->redirect();
        }

    }

    /**
     * Show hotel for editing.
     *
     * @param Request $request
     * @param Model   $hotel
     *
     * @return Response
     */
    public function edit(HotelRequest $request, Hotel $hotel)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('objects::hotel.name'))
            ->view('objects::hotel.edit', true)
            ->data(compact('hotel'))
            ->output();
    }

    /**
     * Update the hotel.
     *
     * @param Request $request
     * @param Model   $hotel
     *
     * @return Response
     */
    public function update(HotelRequest $request, Hotel $hotel)
    {
        try {
            $attributes = $request->all();

            $hotel->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('objects::hotel.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/hotel/' . $hotel->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/hotel/' . $hotel->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the hotel.
     *
     * @param Model   $hotel
     *
     * @return Response
     */
    public function destroy(HotelRequest $request, Hotel $hotel)
    {
        try {

            $hotel->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::hotel.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('objects/hotel/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/hotel/' . $hotel->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple hotel.
     *
     * @param Model   $hotel
     *
     * @return Response
     */
    public function delete(HotelRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::hotel.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('objects/hotel'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/hotel'))
                ->redirect();
        }

    }

    /**
     * Restore deleted hotels.
     *
     * @param Model   $hotel
     *
     * @return Response
     */
    public function restore(HotelRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('objects::hotel.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/objects/hotel'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/hotel/'))
                ->redirect();
        }

    }

}
