<?php

namespace Btravel\Objects\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Objects\Http\Requests\ShipRequest;
use Btravel\Objects\Interfaces\ShipRepositoryInterface;
use Btravel\Objects\Models\Ship;

/**
 * Resource controller class for ship.
 */
class ShipResourceController extends BaseController
{

    /**
     * Initialize ship resource controller.
     *
     * @param type ShipRepositoryInterface $ship
     *
     * @return null
     */
    public function __construct(ShipRepositoryInterface $ship)
    {
        parent::__construct();
        $this->repository = $ship;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Objects\Repositories\Criteria\ShipResourceCriteria::class);
    }

    /**
     * Display a list of ship.
     *
     * @return Response
     */
    public function index(ShipRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Objects\Repositories\Presenter\ShipPresenter::class)
                ->$function();
        }

        $ships = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('objects::ship.names'))
            ->view('objects::ship.index', true)
            ->data(compact('ships', 'view'))
            ->output();
    }

    /**
     * Display ship.
     *
     * @param Request $request
     * @param Model   $ship
     *
     * @return Response
     */
    public function show(ShipRequest $request, Ship $ship)
    {

        if ($ship->exists) {
            $view = 'objects::ship.show';
        } else {
            $view = 'objects::ship.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('objects::ship.name'))
            ->data(compact('ship'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new ship.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ShipRequest $request)
    {

        $ship = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('objects::ship.name')) 
            ->view('objects::ship.create', true) 
            ->data(compact('ship'))
            ->output();
    }

    /**
     * Create new ship.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ShipRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $ship                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('objects::ship.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/ship/' . $ship->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/objects/ship'))
                ->redirect();
        }

    }

    /**
     * Show ship for editing.
     *
     * @param Request $request
     * @param Model   $ship
     *
     * @return Response
     */
    public function edit(ShipRequest $request, Ship $ship)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('objects::ship.name'))
            ->view('objects::ship.edit', true)
            ->data(compact('ship'))
            ->output();
    }

    /**
     * Update the ship.
     *
     * @param Request $request
     * @param Model   $ship
     *
     * @return Response
     */
    public function update(ShipRequest $request, Ship $ship)
    {
        try {
            $attributes = $request->all();

            $ship->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('objects::ship.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('objects/ship/' . $ship->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/ship/' . $ship->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the ship.
     *
     * @param Model   $ship
     *
     * @return Response
     */
    public function destroy(ShipRequest $request, Ship $ship)
    {
        try {

            $ship->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::ship.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('objects/ship/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('objects/ship/' . $ship->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple ship.
     *
     * @param Model   $ship
     *
     * @return Response
     */
    public function delete(ShipRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('objects::ship.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('objects/ship'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/ship'))
                ->redirect();
        }

    }

    /**
     * Restore deleted ships.
     *
     * @param Model   $ship
     *
     * @return Response
     */
    public function restore(ShipRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('objects::ship.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/objects/ship'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/objects/ship/'))
                ->redirect();
        }

    }

}
